{$A+,B-,D-,E+,F-,I+,L-,N-,O-,R-,S+,V+}
{$M $1000,$1000,$1000}

PROGRAM ISETENV;

{
 modified by Jim Scarletta - for using with pipes!,
 use /f param to fill spaces with '_' chars
 }

USES
    Dos,
    Useful;


TYPE
    Psp_Data = ARRAY [0..$FF] OF BYTE;
    Psp_Type = ^Psp_Data;


CONST
     Env_Ofs = $2C;   { Offset in PSP for Environment Segment    }
     Psp_Ofs = $16;   { Offset in PSP for Parent Psp Segemnt     }
     Env_Ptr = $03;   { Offset in PSP-1 for Environment Segement }
     Env_Inc = $01;   { Offset to add to Word at Env_Ptr         }
     Siz_Ofs = $03;   { Offset in Env-1 for Environment Size     }

     Para_To_Bytes = 16;


PROCEDURE Find_Master_Psp (VAR Psp : WORD);
VAR
   Next_Psp : WORD;
BEGIN { Find_Master_Psp }
     REPEAT
           Next_Psp := MEMW[Psp:Psp_Ofs];  { Follow the thread }
           IF Next_Psp <> Psp              { Until recursive   }
           THEN Psp := Next_Psp
           ELSE EXIT;
     UNTIL FALSE;
END; { Find_Master_Psp }


PROCEDURE Get_Environment (Psp : WORD; VAR Env, Size : WORD);
BEGIN { Get Environment }
     IF MEMW [Psp:Env_Ofs] = 0   { Did you expect consistency from MS }
     THEN Env := MEMW [Psp-1:Env_Ptr] + Env_Inc + Psp
     ELSE Env := MEMW [Psp:Env_Ofs];
     Size := MEMW [Env-1:Siz_Ofs] * Para_To_Bytes;
END; { Get Environment }


TYPE
    Result_Type = (Succeeded, No_Match, No_Room, Error, Trying);

FUNCTION Modify_Env (Env, Size : WORD;
                     VAR Variable, Value) : Result_Type;

VAR
   Pos_In_Env : WORD;
   Result     : Result_Type;
   Var_Name   : STRING ABSOLUTE Variable;
   Var_Value  : STRING ABSOLUTE Value;
   Addition   : STRING;

FUNCTION Char_In_Environment (Ch : CHAR) : BOOLEAN;
BEGIN { Char_In_Environment }
     Char_In_Environment := FALSE;
     WHILE MEM[Env:Pos_In_Env] <> ORD(Ch)
     DO BEGIN
        INC (Pos_In_Env);
        IF (Pos_In_Env > Size)
        THEN EXIT;
     END;
     Char_In_Environment := TRUE;
END; { Char_In_Environment }

PROCEDURE Check_And_Substitute;
VAR
   Dest, Src, Final  : WORD;
   Pos_In_Var        : WORD;

BEGIN { Check_And_Substitute }
     IF Result = No_Match
     THEN BEGIN
          Dest := Pos_In_Env;
     END
     ELSE BEGIN
          Dest := Pos_In_Env;
          FOR Pos_In_Var := 1 TO LENGTH (Var_Name)
          DO IF MEM[Env:Pos_In_Env] = ORD(Var_Name [Pos_In_Var])
             THEN BEGIN
                  INC (Pos_In_Env);
                  IF Pos_In_Env > Size
                  THEN BEGIN
                       Result := No_Match;
                       EXIT;                  { Past Environment }
                  END;
             END
             ELSE EXIT;                               { No Match }

          IF MEM[Env:Pos_In_Env] <> ORD ('=')
          THEN EXIT;                             { Partial Match }

          IF NOT Char_In_Environment (#0)
          THEN BEGIN
               Result := Error;
               EXIT;                      { Environment  Corrupt }
          END;
          Src := Pos_In_Env+1;

          WHILE MEMW[Env:Pos_In_Env] <> 0
          DO BEGIN
             IF Pos_In_Env > Size
             THEN BEGIN
                  Result := Error;
                  EXIT;                   { Environment  Corrupt }
             END;
             INC (Pos_In_Env);
          END;
          Final := Pos_In_Env+1;
                                               { Remove Variable }
          Disable_Interrupt;              { Lock from nasty TSRs }
          MOVE (MEM[Env:Src], MEM[Env:Dest], (Final-Src+1));
          Enable_Interrupt;

         Dest := Dest+Final-Src;
     END;

     IF LENGTH(Var_Value) = 0
     THEN BEGIN
          Result := Succeeded;                   { Deletion Only }
          EXIT;
     END;

     Addition := Var_Name + '=' + Var_Value + #0 + #0;
     IF (Dest + LENGTH(Addition)) > Size
     THEN BEGIN
          Result := No_Room;
          EXIT;                                 { No Room in Env }
     END;
                                                  { Add Variable }
     Disable_Interrupt;                   { Lock from nasty TSRs }
     MOVE (Addition[1], MEM[Env:Dest], LENGTH(Addition));
     Enable_Interrupt;
     Result := Succeeded;
END; { Check_And_Substitute }

PROCEDURE Get_Next_Variable;
BEGIN { Get_Next_Variable }
     IF Char_In_Environment (#0)
     THEN BEGIN
          INC (Pos_In_Env);
          IF MEM[Env:Pos_In_Env] = 0   { Env ends with 0000 }
          THEN Result := No_Match
          ELSE;
     END
     ELSE Result := Error;
END; { Get_Next_Variable }

BEGIN { Modify_Env }
     Pos_In_Env := 0;
     Result := Trying;
     Check_And_Substitute;
     WHILE Result = Trying
     DO BEGIN
        Get_Next_Variable;
        IF (Result = Trying) OR (Result = No_Match)
        THEN Check_And_Substitute;
     END;
     Modify_Env := Result;
END; { Modify_Env }


VAR
   Psp, Env : WORD;
   Size     : WORD;
   Version  : WORD;
   Variable : ST40;
   Value    : string;
   i        : integer;
   parm     : string;

BEGIN { SETENV }
     Version := DosVersion;
     IF LO (Version) < 3
     THEN WRITE (BEL, 'Intended for Version 3.XX, so please exercise caution',
                 CR, LF, BEL);

     Psp := PrefixSeg;
     Find_Master_Psp (Psp);
     Get_Environment (Psp,Env,Size);
{
     WRITE (CR, LF, 'SETENV - Test for environment modification, R. Gupta',
            CR, LF, CR, LF,
            'Dos Version      : ', LO(Version), '.', HI(Version), CR, LF,
            'Environment Size : ', Size, CR, LF,
            'Environment Adr  : ', Get_Word_Str (Env), CR, LF, CR, LF);
}
     Variable := ParamStr(1);
     parm := ParamStr(2);
     for i:=1 to length(parm) do parm[i]:=upcase(parm[i]);
     Readln(Value);
     if (parm='/P') or (parm='/PF')  or (parm='/FP') then
       for i:=1 to length(Value) do
         if value[i] in ['>','<','|'] then value[i]:='_';
     if (parm='/F') or (parm='/PF')  or (parm='/FP') then
       for i:=1 to length(Value) do
         if value[i] in [' '] then value[i]:='_';
     Convert_UC (Variable);
     IF LENGTH(Variable) = 0
     THEN BEGIN
          WRITE ('Syntax is  <command> | ISETENV VARIABLE ', CR, LF,
                 'to do what the following does in COMMAND.COM', CR, LF,
                 'SET VARIABLE=VALUE', CR, LF, CR, LF);
          HALT (1);
     END;

     CASE Modify_Env (Env,Size,Variable,Value) OF
          Succeeded : begin end; {WRITE ('Succeeded');}
          No_Match  : halt(1); {WRITE ('Variable Not found');}
          No_Room   : halt(1); {WRITE ('Not enough Space in the Environment');}
          Error     : halt(1); {WRITE ('Environment Corrupt');}
          ELSE        halt(1); {WRITE ('Internal Error');}
     END;
END. { SETENV }
